unit uEditarEnganches;

interface

uses
  {$IFDEF WINDOWS}
   Windows,
   {$ELSE}
   LCLType,
   {$ENDIF}
 Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, utilidades, Grids, uEstados, StdCtrls, uCosa, uConstantesSimSEE,
  uSalasDeJuego, xMatDefs, uOpcionesSimSEEEdit, uBaseEditores;

resourcestring
  mesValoresEnTablaNumericos = 'Los valores introducidos en la tabla deben ser numéricos';
  mesValoresEnTablaNumericosDiscretos = 'Los valores introducidos en la '
    +'tablas de variables discretas deben ser numéricos y discretos.';

type
  TEditarEnganches = class(TBaseEditores)
    sgVarsCont: TStringGrid;
    LVarsCont: TLabel;
    LVarsDisc: TLabel;
    sgVarsDisc: TStringGrid;
    BGuardar: TButton;
    BCancelar: TButton;
    procedure BGuardarClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure BCancelarClick(Sender: TObject);
  private
    cf: TAdminEstados;
    vars_continuas_nombres, vars_continuas_unidades: array of string;
    vars_discretas_nombres, vars_discretas_unidades: array of string;
    function validarFormulario: boolean;
    function buscarValorContinuo(enganchesContinuos: TListaDeCosas; nombre, unidades: String; var valor : NReal): boolean;
    function buscarValorDiscreto(enganchesDiscretos: TListaDeCosas; nombre, unidades: String; var valor : Integer): boolean;
  public
    sala: TSalaDeJuego;

    Constructor Create(AOwner: TComponent; nArch: String; sala: TSalaDeJuego); reintroduce;
  end;

var
  EditarEnganches: TEditarEnganches;

implementation
  {$R *.lfm}

function TEditarEnganches.validarFormulario: boolean;
var
  i: Integer;
  res: boolean;
begin
  res:= true;
  try
    for i:= 1 to sgVarsCont.RowCount - 1 do
      StrToFloat(sgVarsCont.Cells[3, i]);
  except
    on EConvertError do
    begin
      ShowMessage(mesValoresEnTablaNumericos);
      res:= false;
    end;
  end;

  try
    for i:= 1 to sgVarsDisc.RowCount - 1 do
      StrToInt(sgVarsDisc.Cells[3, i]);
  except
    on EConvertError do
    begin
      ShowMessage(mesValoresEnTablaNumericosDiscretos);
      res:= false;
    end;
  end;

  result:= res;
end;

function TEditarEnganches.buscarValorContinuo(enganchesContinuos: TListaDeCosas; nombre, unidades: String; var valor : NReal): boolean;
var
  i: Integer;
  encontre: boolean;
  ae: TEngancheCFReal;
begin
  encontre:= false;
  for i:= 0 to enganchesContinuos.Count - 1 do
  begin
//    writeln( 'nombre: ('+nombre+'), ec_nombre: ('+TEngancheCFReal(enganchesContinuos[i]).nombreVar+')');
    ae:=  TEngancheCFReal(enganchesContinuos[i]);
    if ( nombre = ae.nombreVar )  and ( unidades = ae.unidades ) then
    begin
      valor:= ae.valor;
      encontre:= true;
      break;
    end;
  end;
  result:= encontre;
end;

function TEditarEnganches.buscarValorDiscreto(enganchesDiscretos: TListaDeCosas; nombre, unidades: string; var valor : Integer): boolean;
var
  i: Integer;
  encontre: boolean;
  ae: TEngancheCFEntero;
begin
  encontre:= false;
  for i:= 0 to enganchesDiscretos.Count - 1 do
  begin
    ae:= TEngancheCFEntero(enganchesDiscretos[i]);
    if ( nombre = ae.nombreVar) and ( unidades = ae.unidades ) then
    begin
      valor:= ae.valor;
      encontre:= true;
      break;
    end;
  end;
  result:= encontre;
end;

procedure TEditarEnganches.BCancelarClick(Sender: TObject);
begin
  setlength( vars_continuas_nombres, 0 );
  setlength( vars_continuas_unidades, 0 );
  setlength( vars_discretas_nombres, 0 );
  setlength( vars_discretas_unidades, 0 );
  ModalResult:= mrCancel;
end;

procedure TEditarEnganches.BGuardarClick(Sender: TObject);
var
  i: Integer;
  val_s: string;
begin
  if validarFormulario then
  begin
    if sala.enganchesContinuos <> NIL then
      sala.enganchesContinuos.Free;
    sala.enganchesContinuos:= TListaDeCosas.Create(capa, 'EnganchesContinuos');
    if sala.enganchesDiscretos <> NIL then
      sala.enganchesDiscretos.Free;
    sala.enganchesDiscretos:= TListaDeCosas.Create(capa, 'EnganchesDiscretos');

    for i:= 0 to high( vars_continuas_nombres ) do
    begin
      val_s:= sgVarsCont.Cells[3, i + 1];
      sala.enganchesContinuos.add(TEngancheCFReal.Create(capa, vars_continuas_nombres[i], vars_continuas_unidades[i],
                                                         StrToFloat(val_s)));
    end;
    for i:= 0 to high( vars_discretas_nombres ) do
    begin
      val_s:= sgVarsDisc.Cells[3, i + 1];
      sala.enganchesDiscretos.add(TEngancheCFEntero.Create(capa, vars_discretas_nombres[i], vars_discretas_unidades[i],
                                                           StrToInt(val_s)));
    end;
    ModalResult:= mrOk;
  end;
end;

Constructor TEditarEnganches.Create(AOwner: TComponent; nArch: String; sala: TSalaDeJuego);
var
  i: Integer;
  valReal: NReal;
  valEntero: Integer;
  var_nombre, var_unidades: string;

begin
  inherited Create(AOwner, nil, sala );
  self.sala:= sala;
  cf:= TAdminEstados.CreateLoadFromArchi(nArch);

  setlength( vars_continuas_nombres, cf.nVarsContinuas );
  setlength( vars_continuas_unidades, cf.nVarsContinuas );
  if cf.nVarsContinuas > 0 then
  begin
    sgVarsCont.RowCount:= cf.nVarsContinuas + 1;
    sgVarsCont.Cells[0, 0]:= 'Variable';
    sgVarsCont.Cells[1, 0]:= 'Limites (Inf, Sup)';
    sgVarsCont.Cells[2, 0]:= 'Valor Medio';
    sgVarsCont.Cells[3, 0]:= 'Valor';
    for i:= 0 to cf.nVarsContinuas - 1 do
    begin
      var_nombre:=cf.xr_def[i].nombre;
      var_unidades:= cf.xr_def[i].unidades;
      vars_continuas_nombres[i]:= var_nombre;
      vars_continuas_unidades[i]:= var_unidades;

      sgVarsCont.Cells[0, i + 1]:= var_nombre + ' [' + var_unidades + ']';
      sgVarsCont.Cells[1, i + 1]:= '(' + FloatToStr(cf.xr_def[i].x[0]) + ', ' + FloatToStr(cf.xr_def[i].x[high(cf.xr_def[i].x)]) + ')';
      sgVarsCont.Cells[2, i + 1]:= FloatToStr(cf.xr_def[i].xmed);

      if (sala.enganchesContinuos <> NIL) and buscarValorContinuo(sala.enganchesContinuos, var_nombre, var_unidades, valReal) then
        sgVarsCont.Cells[3, i + 1]:= FloatToStr(valReal)
      else
        sgVarsCont.Cells[3, i + 1]:= FloatToStr(cf.xr_def[i].xmed);
    end;
  end
  else
  begin
    sgVarsCont.RowCount:= 0;
    LVarsCont.Visible:= false;
    sgVarsCont.Visible:= false;

    for i:= 0 to ControlCount -1 do
      if Controls[i].Top >= (sgVarsCont.Top + sgVarsCont.Height) then
        controls[i].Top:= Controls[i].Top - ((sgVarsCont.Top - LVarsCont.Top) + sgVarsCont.Height);
  end;


  setlength( vars_discretas_nombres, cf.nVarsDiscretas );
  setlength( vars_discretas_unidades, cf.nVarsDiscretas );
  if cf.nVarsDiscretas > 0 then
  begin
    sgVarsDisc.RowCount:= cf.nVarsDiscretas + 1;
    sgVarsDisc.Cells[0, 0]:= 'Variable';
    sgVarsDisc.Cells[1, 0]:= 'Limites (Inf, Sup)';
    sgVarsDisc.Cells[2, 0]:= 'Valor Mínimo';
    sgVarsDisc.Cells[3, 0]:= 'Valor';
    for i:= 0 to cf.nVarsDiscretas - 1 do
    begin
      var_nombre:=cf.xd_def[i].nombre;
      var_unidades:= cf.xd_def[i].unidades;
      vars_discretas_nombres[i]:= var_nombre;
      vars_discretas_unidades[i]:= var_unidades;

      sgVarsDisc.Cells[0, i + 1]:= var_nombre + ' [' + var_unidades + ']';
      sgVarsDisc.Cells[1, i + 1]:= '(' + IntToStr(cf.xd_def[i].x[0]) + ', ' + IntToStr(cf.xd_def[i].x[high(cf.xd_def[i].x)]) + ')';
      sgVarsDisc.Cells[2, i + 1]:= IntToStr(cf.xd_def[i].x[0]);

      if (sala.enganchesDiscretos <> NIL) and buscarValorDiscreto(sala.enganchesDiscretos, var_nombre, var_unidades, valEntero) then
        sgVarsDisc.Cells[3, i + 1]:= FloatToStr(valEntero)
      else
        sgVarsDisc.Cells[3, i + 1]:= FloatToStr(cf.xd_def[i].x[0]);
    end;
  end
  else
  begin
    sgVarsDisc.RowCount:= 0;
    LVarsDisc.Visible:= false;
    sgVarsDisc.Visible:= false;

    for i:= 0 to ControlCount -1 do
      if Controls[i].Top >= (sgVarsDisc.Top + sgVarsDisc.Height) then
        controls[i].Top:= Controls[i].Top - ((sgVarsDisc.Top - LVarsDisc.Top) + sgVarsDisc.Height);
  end;
  cf.Free;
  for i:= 0 to sgVarsCont.ColCount - 2 do
    utilidades.AutoSizeCol(sgVarsCont, i);
  if (sgVarsDisc.ColCount > 0) then //si no hay valores se rompe por index -1 fix 16/5/2011
    sgVarsCont.ColWidths[sgVarsCont.ColCount - 1]:= 150;
  utilidades.AutoSizeTable( sgVarsCont, maxAnchoTablaEnorme, maxAlturaTablaMediana, TSimSEEEditOptions.getInstance.deshabilitarScrollHorizontalEnListados);

  for i:= 0 to sgVarsDisc.ColCount - 2 do
    utilidades.AutoSizeCol(sgVarsDisc, i);
  if (sgVarsDisc.ColCount > 0) then //si no hay valores se rompe por index -1 fix 16/5/2011
    sgVarsDisc.ColWidths[sgVarsDisc.ColCount - 1]:= 150;
  utilidades.AutoSizeTable( sgVarsDisc, maxAnchoTablaEnorme, maxAlturaTablaMediana, TSimSEEEditOptions.getInstance.deshabilitarScrollHorizontalEnListados);

  self.ClientWidth:= sgVarsCont.Width;
  self.ClientHeight:= LVarsCont.Height + sgVarsCont.Height + LVarsDisc.Height + sgVarsDisc.Height + BGuardar.Height;
end;

procedure TEditarEnganches.FormResize(Sender: TObject);
begin
  BGuardar.Left:= Self.Width div 4 - BGuardar.Width div 2;
  BCancelar.Left:= Self.Width * 3 div 4 - BCancelar.Width div 2;
end;

end.
